#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE
 # Copyright (C) 2021 KylinSoft Co., Ltd.
 #
 # Authors:
 #  Yang Min yangmin@kylinos.cn
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import GLib

import os
import dbus
import signal
import dbus.service
import dbus.mainloop.glib

import subprocess


DBUSNAME = "org.ukui.ukcc.system"
ACTION = "org.ukui.ukcc.system.action"

class DemoException(dbus.DBusException):
    _dbus_error_name = "org.ukui.ukcc.system.Exception"

class PolicyKitService(dbus.service.Object):
    '''A D-BUS service that uses PolicyKit for authorization.'''

    def _check_permission(self, sender, action):
        if not sender:
            raise ValueError('sender == None')
        kit = dbus.SystemBus().get_object('org.freedesktop.PolicyKit1', '/org/freedesktop/PolicyKit1/Authority')
        kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit1.Authority')
        (granted, _, details) = kit.CheckAuthorization(('system-bus-name', {'name': sender}), action, {}, dbus.UInt32(1), '', timeout=600)
        return granted

class TestObject(PolicyKitService):
    INTERFACE = "org.example.interface"
    def __init__(self, bus, mainloop):
        #dbus.service.Object.__init__(self, bus, "/MyObject")
        self.bus = bus
        self.bus_name = dbus.service.BusName(DBUSNAME, bus)

        PolicyKitService.__init__(self, self.bus_name, "/")
        self.mainloop = mainloop

    @dbus.service.method(INTERFACE,
                         in_signature = 's', out_signature = 'as')
    def HelloWorld(self, hello_message):
        print(str(hello_message))
        return ["Hello", "from example-service.py", "with unique name",
                bus.get_unique_name()]

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = 'ss')
    def GetTuple(self):
        return ("Hello Tuple", " from example-service.py")

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = '')
    def RaiseException(self):
        raise DemoException("The RaiseException method does what you might "
                            'expect')

    @dbus.service.method(INTERFACE,
                         in_signature = 'ssssi', out_signature = 's', sender_keyword='sender')
    def CreateNewUser(self, name, fullname, pwd, face, actype, sender=None):
        status = self._check_permission(sender, ACTION)
        if not status:
            return "Not authorized"

        pbus = dbus.SystemBus()
        eth0 = pbus.get_object("org.freedesktop.Accounts", "/org/freedesktop/Accounts")
        eth0_iface = dbus.Interface(eth0, dbus_interface="org.freedesktop.Accounts")
        try:
            obp = eth0_iface.CreateUser(name, fullname, actype)
        except dbus.DBusException as e:
            pass
            #print("client:", str(e))
        else:
            eth1 = pbus.get_object("org.freedesktop.Accounts", obp)
            eth1_iface = dbus.Interface(eth1, dbus_interface="org.freedesktop.Accounts.User")
            # set passwd
            args = ["/usr/bin/changeOtherUserPwd", name, pwd]
            pipe = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            output = pipe.stdout.readlines()

            # set face
            eth1_iface.SetIconFile(face)
        return str(obp)

    @dbus.service.method(INTERFACE,
                         in_signature = 'xsb', out_signature = 's', sender_keyword='sender')
    def DeleteExistUser(self, id, name, removeFiles, sender=None):
        status = self._check_permission(sender, ACTION)
        if not status:
            return "Not authorized"

        pbus = dbus.SystemBus()
        eth0 = pbus.get_object("org.freedesktop.Accounts", "/org/freedesktop/Accounts")
        eth0_iface = dbus.Interface(eth0, dbus_interface="org.freedesktop.Accounts")
        try:
            eth0_iface.DeleteUser(id, removeFiles)
        except dbus.DBusException as e:
            pass
            #print("client:", str(e))
        else:
            ##删除用户生物特征
            df = "/home/" + name + "/.biometric_auth/UUID"
            if os.path.exists(df):
                os.remove(df)
            return "Comlete"

    @dbus.service.method(INTERFACE, in_signature = 'ss', out_signature = 's', sender_keyword='sender')
    def ChangeOtherUserPwd(self, name, pwd, sender=None):
        status = self._check_permission(sender, ACTION)
        if not status:
            return "Not authorized"

        args = ["/usr/bin/changeOtherUserPwd", name, pwd]
        pipe = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = pipe.stdout.readlines()

        return "Complete"

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = '')
    def Exit(self):
        self.mainloop.quit()

if __name__ == '__main__':
    os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"

    dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)
    mainloop = GLib.MainLoop()

    bus = dbus.SystemBus()
    signal.signal(signal.SIGINT, lambda : mainloop.quit())
    obj = TestObject(bus, mainloop)

    print("Running ukcc_system_service...\n")
    mainloop.run()
    print("Ending ukcc_system_service...\n")
